from typing import Any

import requests
from requests import Response


def make_jwt_request(signed_jwt: Any, url: Any) -> Response:
    """Makes an authorized request to the endpoint"""
    headers = {
        "Authorization": "Bearer {}".format(signed_jwt.decode("utf-8")),
        "content-type": "text/html",
    }

    response = requests.get(url, headers=headers)
    return response



import os
import time
from typing import Any, Optional

from fluidly.auth.jwt import generate_jwt
from fluidly.auth.jwt_requests import make_jwt_request
from fluidly.structlog import base_logger


class UserPermissionsRequestException(Exception):
    pass


class UserPermissionsPayloadException(Exception):
    pass


def get_fluidly_api_url(fluidly_api_url: Optional[str] = None) -> str:
    if not fluidly_api_url:
        fluidly_api_url = os.getenv("FLUIDLY_API_URL")

    if not fluidly_api_url:
        raise ValueError("Please provide FLUIDLY_API_URL")
    return fluidly_api_url


def check_permissions(original_payload: Any, request_url: str, **kwargs: Any) -> bool:
    start = time.time()
    signed_jwt = generate_jwt(original_payload)
    try:
        response = make_jwt_request(signed_jwt, request_url)
    except Exception:
        raise UserPermissionsRequestException()

    logger = base_logger.get_logger()
    end = time.time()

    try:
        response_json = response.json()

        authorised = response.status_code == 200 and response_json.get("grantAccess")
        if not authorised:
            logger.warning(
                "Authorisation failed",
                response_json=response_json,
                status_code=response.status_code,
                response_headers=response.headers,
                url=request_url,
                original_payload=original_payload,
                duration=end - start,
                **kwargs,
            )
            return False
        logger.info(
            "Called user permissions",
            response_json=response_json,
            status_code=response.status_code,
            url=request_url,
            duration=end - start,
        )
        return True
    except Exception:
        logger.warning(
            "Authorisation failed",
            status_code=response.status_code,
            response_headers=response.headers,
            url=request_url,
            original_payload=original_payload,
            exc_info=True,
            duration=end - start,
        )
        raise UserPermissionsPayloadException()


def check_user_permissions(
    original_payload: Any, connection_id: str, fluidly_api_url: Optional[str] = None
) -> bool:
    return check_permissions(
        original_payload,
        f"{get_fluidly_api_url(fluidly_api_url)}/v1/user-permissions/connections/{connection_id}",
        connection_id=connection_id,
    )


def check_admin_permissions(
    original_payload: Any, fluidly_api_url: Optional[str] = None
) -> bool:
    return check_permissions(
        original_payload,
        f"{get_fluidly_api_url(fluidly_api_url)}/v1/user-permissions/admin",
    )



import json
from base64 import b64encode


def request_headers(
    if_modified_since=None, service_account=False, email=None, name=None
):
    # Claims attribute is in string format in the header provided by Cloud Endpoints
    claims = {
        "https://api.fluidly.com/internal_metadata": {
            "isServiceAccount": service_account
        },
        "https://api.fluidly.com/user_metadata": {},
        "https://api.fluidly.com/app_metadata": {"userId": "1"},
        "iss": "https://fluidly.eu.auth0.com/",
        "sub": "auth0|5cb644d401c03f119f1ed772",
        "aud": ["https://api.fluidly.com", "https://fluidly.eu.auth0.com/userinfo"],
        "iat": 1555519624,
        "exp": 1555526824,
        "azp": "5HIxj76CRK3QCxUa2KlUwJw7vBCly9e5",
        "scope": "openid email",
    }

    if email is not None:
        claims["https://api.fluidly.com/email"] = email

    if name is not None:
        claims["https://api.fluidly.com/name"] = name

    endpoint_header = json.dumps(
        {
            "claims": json.dumps(claims),
            "issuer": "https://fluidly.eu.auth0.com/",
            "id": "auth0|5cb644d401c03f119f1ed772",
        }
    )
    headers = {
        "X-Endpoint-Api-Userinfo": b64encode(endpoint_header.encode("utf-8")),
        "Content-Type": "application/json",
    }
    if if_modified_since is not None:
        headers["If-Modified-Since"] = if_modified_since

    return headers



import os
import time
from typing import Any, Optional, Union

from google.auth import crypt, jwt
from google.oauth2.service_account import Credentials

audience: str = "https://api.fluidly.com"


def get_service_account_and_signer(
    path: Union[str, None], info: Union[str, None]
) -> Any:
    try:
        if path is not None:
            return Credentials.from_service_account_file(
                path
            ).service_account_email, crypt.RSASigner.from_service_account_file(path)
        elif info is not None:
            return Credentials.from_service_account_info(
                info
            ).service_account_email, crypt.RSASigner.from_service_account_info(info)
    except FileNotFoundError or AttributeError:
        raise Exception("Credentials must be a path or json")


def generate_jwt(
    claims: Any,
    google_application_credentials: Optional[Any] = None,
    google_application_credentials_info: Optional[Any] = None,
) -> Any:
    """Generates a signed JSON Web Token using a Google API Service Account."""

    if os.getenv("AUTH0_JWT_TOKEN"):
        return os.getenv("AUTH0_JWT_TOKEN")

    if not google_application_credentials:
        google_application_credentials = os.getenv("GOOGLE_APPLICATION_CREDENTIALS")
    if not google_application_credentials_info:
        google_application_credentials_info = os.getenv("GOOGLE_CREDENTIALS")

    if not google_application_credentials and not google_application_credentials_info:
        raise ValueError(
            "Please provide GOOGLE_APPLICATION_CREDENTIALS or GOOGLE_CREDENTIALS"
        )

    now = int(time.time())

    sa_email, signer = get_service_account_and_signer(
        google_application_credentials, google_application_credentials_info
    )

    payload = {
        "iat": now,
        "exp": now + 3600,
        # iss must match 'issuer' in the security configuration in your
        # swagger spec (e.g. service account email). It can be any string.
        "iss": sa_email,
        # aud must be either your Endpoints service name, or match the value
        # specified as the 'x-google-audience' in the OpenAPI document.
        "aud": audience,
        # sub and email should match the service account's email address
        "sub": sa_email,
        "email": sa_email,
    }

    claims.update(payload)

    jwt_string = jwt.encode(signer, claims)

    return jwt_string
